#ifndef _BT_MISC_H
#define _BT_MISC_H

#include <stdbool.h>
#include <stdint.h>
#include <ck_hs.h>

#define BT_STATIC_ARRAY_SIZE(x) (sizeof(x) / sizeof(*(x)))
#define BT_ALIGN(S, A) (((A) - ((S) % (A))) % (A))
#define BT_PAD(S, P) (((S) + (P) - 1) & ~((P) - 1))
#define BT_POWER2(X) (!((X) & ((X) - 1)))
#define BT_OFFSETOF(T, F) ((ptrdiff_t) &((T *)0)->F)

#define BT_CC_PACKED		__attribute__((packed))
#define BT_CC_ALWAYS_INLINE	__attribute__((always_inline))

#ifdef STAILQ_FOREACH_SAFE
#define BT_STAILQ_FOREACH_SAFE STAILQ_FOREACH_SAFE
#else
#define BT_STAILQ_FOREACH_SAFE(ITEM, LIST, LINK, TEMP)			\
	for ((ITEM) = STAILQ_FIRST((LIST));				\
	    ((ITEM) != NULL) && ((TEMP) = STAILQ_NEXT((ITEM), LINK), 1);\
	    (ITEM) = (TEMP))
#endif

#ifdef SLIST_FOREACH_SAFE
#define BT_SLIST_FOREACH_SAFE SLIST_FOREACH_SAFE
#else
#define BT_SLIST_FOREACH_SAFE(var, head, field, tvar)		\
        for ((var) = SLIST_FIRST((head));			\
        (var) && ((tvar) = SLIST_NEXT((var), field), 1);	\
        (var) = (tvar))
#endif

#ifndef max
#define max(a, b) BT_MAX(a, b)
#endif

#ifndef min
#define min(a, b) BT_MIN(a, b)
#endif

#define BT_MAX(a, b)					\
	({						\
		__typeof__(a) _a = (a);			\
		__typeof__(b) _b = (b);			\
		_a > _b ? _a : _b;			\
	})

#define BT_MIN(a, b)					\
	({						\
		__typeof__(a) _a = (a);			\
		__typeof__(b) _b = (b);			\
		_b > _a ? _a : _b;			\
	})

/* Be sure to call bt_misc_init_rand_state before this. */
bool bt_ck_hs_init_capacity(ck_hs_t *hs, unsigned int mode,
    ck_hs_hash_cb_t *hash_fn, ck_hs_compare_cb_t *compare_fn,
    unsigned long capacity);

/* Be sure to call bt_misc_init_rand_state before this. */
static inline bool
bt_ck_hs_init(ck_hs_t *hs, unsigned int mode, ck_hs_hash_cb_t *hash_fn,
    ck_hs_compare_cb_t *compare_fn)
{
	return bt_ck_hs_init_capacity(hs, mode, hash_fn, compare_fn, 64);
}

/*
 * Attempts to get a seed from the system. Returns false and logs if it fails.
 */
bool bt_misc_get_rand_seed(uint64_t *seed);

/*
 * Must call this before any of the bt_ck_hs_init helper functions.
 */
void bt_misc_init_rand_state(uint64_t seed);

struct bt_variable;
struct bt_error;

/* DWARF-based substitute for sys/queue.h LIST_FOREACH. */
typedef bool list_foreach_cb(struct bt_variable *, void *, struct bt_error *);
bool bt_list_foreach(struct bt_variable *lh, const char *le_name,
    list_foreach_cb *cb, void *arg, struct bt_error *error);

/* DWARF-based substitute for sys/queue.h TAILQ_FOREACH. */
typedef bool tqh_foreach_cb(struct bt_variable *, void *, struct bt_error *);
bool bt_tailq_foreach(struct bt_variable *tqh, const char *tqe_name,
    tqh_foreach_cb *cb, void *arg, struct bt_error *error);

#endif /* _BT_MISC_H */
