#ifndef _BT_TYPE_CACHE_H
#define _BT_TYPE_CACHE_H

#include <stdbool.h>
#include <stdlib.h>

#include "ac.h"
#include "array.h"

#define BT_TC_VERSION (1U)

struct bt_tc_section {
	unsigned char *start;
	unsigned int n_entry;
};

struct bt_asset;
struct bt_bf;

struct bt_tc {
	unsigned char *data;

	struct bt_tc_section files;
	struct bt_tc_section cus;
	struct bt_tc_section entries;

	struct {
		char *data;
		unsigned int length;
	} strtab;

	struct bt_asset *asset;
	struct bt_bf **bfs;
};
typedef struct bt_tc bt_tc_t;

struct bt_dwarf_die;

/* Initialize type cache from on-disk format. */
bool bt_tc_init(bt_tc_t *, struct bt_asset *, void *, size_t);
void bt_tc_close(bt_tc_t *);

/*
 * If this returns true, specification data was written to the specification
 * member of the provided DIE struct.
 */
bool bt_tc_lookup(const bt_tc_t *, struct bt_dwarf_die *);

/* Structs and functions for generating and writing type cache data */
struct bt_tc_context;
struct bt_so_resolver;

struct bt_tc_context *bt_tc_context_init(void);
void bt_tc_context_destroy(struct bt_tc_context *);

/*
 * Add a binary file to the type cache and resolve dependencies
 * using the provided resolver object.
 * If the fourth argument is non-NULL then only CUs matching names in that array
 * will be processed.
 */
bool bt_tc_bf(struct bt_tc_context *, struct bt_bf *bf, const char *,
    struct bt_so_resolver *, BT_ARRAY(cu) *);

/* Functions for writing type cache data to asset file */
size_t bt_tc_size(void *);
bool bt_tc_write(unsigned char *, void *);

/*
 * Interface for iterating over all entries.
 * Primarily useful for debugging purposes.
 */
struct bt_tc_iterator {
	const bt_tc_t *cache;
	unsigned int index;
};

struct bt_tc_iterator_entry {
	const char *signature;
	const char *path;
	unsigned long long cu_offset;
	unsigned long long offset;
};

void bt_tc_iterator_init(struct bt_tc_iterator *,
    const bt_tc_t *);
bool bt_tc_iterator_next(struct bt_tc_iterator *,
    struct bt_tc_iterator_entry *);

#endif /* _BT_TYPE_CACHE_H */
